/*----------------------------------------------------------------------
  SDL2 Benchmark - Roberto Prieto 
  Copyright (C) 2013-2023 MegaStorm Systems
  contact@megastormsystems.com - http://www.megastormsystems.com

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

 ------------------------------------------------------------------------
                                SDL2 Benchmark Utility
    
    - Utility functions

    Changelog:
 
 v1.00 Sept/2015 - First version.
   
 ------------------------------------------------------------------------ */

#include <algorithm>
#include <string>
#include "SDL-Benchmark.h"

char *getCmdOption(char ** begin, char ** end, const std::string & option)
{
    char ** itr = std::find(begin, end, option);
    if(itr != end && ++itr != end) return *itr;
    return 0;
}

int cmdOptionExists(char** begin, char** end, const std::string& option)
{
    return std::find(begin, end, option) != end;
}

int parseAndInformation(int argc, char *argv[], char *szName, SDLBenchParameters *params)
{
    int iTmp;
    char *pBuffer = NULL;

    // Set default values
    params->iNumSprites = NUM_SPRITES;
    params->iNumSprites2 = NUM_SPRITES2;
    params->dBaseTime = BASETIME;
    params->iVideoWidth = VIDEO_WIDTH;
    params->iVideoHeight = VIDEO_HEIGHT;
    params->szIcon = ICON_TEST;
    params->iFillRateMode = 0;
    params->iIntegrated = 0;

    // Parse arguments
    pBuffer = getCmdOption(argv, argv + argc, "-width");
    if(pBuffer)
    {
        iTmp = atoi(pBuffer);
        if(iTmp > 256 && iTmp < 4000) params->iVideoWidth = iTmp;
    }
    pBuffer = getCmdOption(argv, argv + argc, "-height");
    if(pBuffer)
    {
        iTmp = atoi(pBuffer);
        if(iTmp > 256 && iTmp < 4000) params->iVideoHeight = iTmp;
    }
    pBuffer = getCmdOption(argv, argv + argc, "-time");
    if(pBuffer)
    {
        iTmp = atoi(pBuffer);
        if(iTmp > 0 && iTmp < 60) params->dBaseTime = iTmp;
    }
    pBuffer = getCmdOption(argv, argv + argc, "-test");
    if(pBuffer)
    {
        iTmp = atoi(pBuffer);
        if(iTmp > 0 && iTmp < 100000) params->iNumSprites = iTmp;
    }
    pBuffer = getCmdOption(argv, argv + argc, "-test2");
    if(pBuffer)
    {
        iTmp = atoi(pBuffer);
        if(iTmp > 0 && iTmp < 100000) params->iNumSprites2 = iTmp;
    }
    pBuffer = getCmdOption(argv, argv + argc, "-icon");
    if(pBuffer)
    {
        params->szIcon = pBuffer;
    }
    if(cmdOptionExists(argv, argv + argc, "-fillrate")) params->iFillRateMode = 1;

    // Print information and current parameters
    if(cmdOptionExists(argv, argv + argc, "-integrated"))
    {
        params->iIntegrated = 1;
        // Integrated mode        
        printf("----------------------------------------\n");
        printf(" %s\n\n", szName);                
    }    
    else
    {
        // Stand-alone mode
        printf("----------------------------------------------------------------------\n");
        printf(" %s - Roberto Prieto\n", szName);
        printf(" Copyright (C) 2013-2023 MegaStorm Systems\n");
        printf("----------------------------------------------------------------------\n\n");
        printf(" Running at %dx%d resolution (can be set with -width and -height)\n", params->iVideoWidth, params->iVideoHeight);
        printf(" Each test will run for %d seconds (can be set with -time)\n", (int)params->dBaseTime);
        printf(" Tests with %d and %d icons (can be set with -test and -test2)\n", params->iNumSprites, params->iNumSprites2);
        printf(" Using '%s' as icon (can be set with -icon and supports BMP format)\n", params->szIcon);
        if(params->iFillRateMode == 0) printf(" One texture for each sprite (use -fillrate for sharing a single texture)\n");
        else printf(" Single texture shared for all sprites\n");
        puts("");
    }
    return 0;
}
